@@ -5,18 +5,58 @@ from numpy import random
 
 import torch
 import torch.nn as nn
+import torch.nn.functional as F
 
 from einops import rearrange
 
 
 class MHAttention(nn.Module):
     """
-    Multi-head self-attention using einops and custom linear layer.
+    Multi-head self-attention using einops and optionally a custom linear layer.
 
     Forward method assumes q, k and v have the same embedding size and k and v
         are the same shape.
 
     Assumes bias=False and batch_first=True, as God intended.
+
+    Optionally adds various bells and whistles suggested in the
+        literature, including:
+
+        Noam Shazeer's scaled attention per "Attention is All You Need"
+            (https://arxiv.org/abs/1706.03762).
+
+        Max subtract softmax as discussed in "Attention As An RNN"
+            (https://arxiv.org/abs/2405.13956)
+
+        Log-length scaled softmax per "Overcoming a Theoretical Limitation of
+            Self-Attention" (https://arxiv.org/abs/2202.12172).
+
+        Quiet softmax per
+            https://www.evanmiller.org/attention-is-off-by-one.html
+
+    Args:
+        d_model: ...
+        n_heads: ...
+        dropout: ...
+        causal: should a causal mask be applied to the logits before attention
+            is applied? This is standard when using self-attention. Cannot be
+            True if inputs won't be square (e.g. if sequence length for
+            encoder and decoder are different)
+        sequence_length: ...
+        share_kv: ...
+        linear_module: ...
+        max_subtract: if True, the maximum logit value is subtracted from all
+            logits before performing the softmax operation to create a more
+            numerically stable softmax. This is discussed in "Attention As An
+            RNN" (https://arxiv.org/abs/2405.13956).
+        d_model_scale: ...
+        log_length_scale: if True, multiplies logits by the log length of
+            the decoder sequence before performing the softmax operation, as
+            proposed in "Overcoming a Theoretical Limitation of Self-Attention"
+            (https://arxiv.org/abs/2202.12172).
+        quiet: if True, adds 1 to the denominator of the softmax operation,
+            allowing some tokens to attend to no other tokens as described in
+            https://www.evanmiller.org/attention-is-off-by-one.html.
     """
 
     def __init__(
@@ -26,8 +66,12 @@ class MHAttention(nn.Module):
         dropout=0.0,
         causal=False,
         sequence_length=None,
-        share_kv=True,
+        share_kv=False,
         linear_module: nn.Module = nn.Linear,
+        max_subtract=False,
+        d_model_scale=True,
+        log_length_scale=False,
+        quiet=False,
     ):
         super().__init__()
         if causal:
@@ -57,6 +101,29 @@ class MHAttention(nn.Module):
                 .unsqueeze(0)
                 .unsqueeze(0),
             )
+        self.max_subtract = max_subtract
+        self.d_model_scale = d_model_scale
+        self.log_length_scale = log_length_scale
+        self.quiet = quiet
+
+    @property
+    def _kv_distance(self) -> float:
+        """
+        Calculates the cosine distance between the weight tensors of `self.k_proj`
+            and `self.v_proj`.
+
+        The cosine distance is defined as 1 - cosine_similarity (i.e. a value
+            closer to 0 indicates higher similarity.
+        """
+
+        similarity = F.cosine_similarity(
+            self.k_proj.weight.detach().flatten(),
+            self.v_proj.weight.detach().flatten(),
+            dim=0,
+            eps=1e-8,
+        ).item()
+
+        return 1 - similarity
 
     def forward(self, q, k, v):
         query_batch_size, query_tokens, query_features = q.size()
@@ -82,14 +149,27 @@ class MHAttention(nn.Module):
             v = rearrange(self.v_proj(v), "b t (h d) -> b h t d", h=self.n_heads)
 
         qk_scores = q @ k.transpose(-1, -2)
-        qk_scores /= math.sqrt(self.head_dim)  # scaling
+
+        if self.d_model_scale:
+            qk_scores /= math.sqrt(self.head_dim)  # scaling
+
+        if self.log_length_scale:
+            qk_scores *= math.log(qk_scores.size(0))
+
+        if self.max_subtract:
+            max_scores, _ = torch.max(qk_scores, dim=-1, keepdim=True)
+            qk_scores -= max_scores
 
         # Apply mask if causal (must come before softmax)
         if self.causal:
             qk_scores.masked_fill_(self.mask, float("-inf"))
 
-        qk_scores = torch.softmax(qk_scores, dim=-1)  # softmax
-        qk_scores = self.dropout(qk_scores)  # dropout must come after softmax!
+        # Apply softmax and dropout
+        denominator = torch.sum(torch.exp(qk_scores), dim=-1, keepdim=True)
+        if self.quiet:
+            denominator += 1
+        numerator = torch.exp(qk_scores)
+        qk_scores = self.dropout(numerator / denominator)
 
         output_with_heads = qk_scores @ v
 
@@ -117,6 +197,11 @@ class TransformerBlock(nn.Module):
         mlp_dropout=0.0,
         msa_dropout=0.0,
         causal=False,
+        share_kv=False,
+        max_subtract=False,
+        d_model_scale=True,
+        log_length_scale=False,
+        quiet_attention=False,
         linear_module=nn.Linear,
     ):
         super().__init__()
@@ -128,11 +213,17 @@ class TransformerBlock(nn.Module):
 
         # Submodules for applying attention
         self.layer_norm = nn.LayerNorm(d_model)
+
         self.attn = MHAttention(  # Handles QKV projection
             d_model,
             n_heads,
             dropout=msa_dropout,
             causal=causal,
+            share_kv=share_kv,
+            max_subtract=max_subtract,
+            d_model_scale=d_model_scale,
+            log_length_scale=log_length_scale,
+            quiet=quiet_attention,
             linear_module=linear_module,
         )
 
@@ -161,6 +252,10 @@ class TransformerBlock(nn.Module):
             )
         )
 
+    @property
+    def _kv_distance(self) -> float:
+        return self.attn._kv_distance
+
     def forward(self, x):
         normx = self.layer_norm(x)
         x = x + self.attn(normx, normx, normx)
@@ -187,6 +282,11 @@ class TransformerEncoder(nn.Module):
         msa_dropout=0.0,
         stochastic_depth=0.0,
         causal=False,
+        share_kv=False,
+        max_subtract=False,
+        d_model_scale=True,
+        log_length_scale=False,
+        quiet_attention=False,
         linear_module=nn.Linear,
         bos_tokens=0,
     ):
@@ -220,12 +320,21 @@ class TransformerEncoder(nn.Module):
                     mlp_dropout=mlp_dropout,
                     msa_dropout=msa_dropout,
                     causal=causal,
+                    share_kv=share_kv,
+                    max_subtract=max_subtract,
+                    d_model_scale=d_model_scale,
+                    log_length_scale=log_length_scale,
+                    quiet_attention=quiet_attention,
                     linear_module=linear_module,
                 )
                 for _ in range(n_layers)
             ]
         )
 
+    @property
+    def _kv_distances(self) -> float:
+        return ",".join([str(block._kv_distance) for block in self.blocks])
+
     def forward(self, x):
         if self._bos_tokens:
             x = torch.cat([self._bos.expand(x.size(0), -1, -1), x], dim=1)
