@@ -1,7 +1,7 @@
 from typing import Optional
 
 from .transformer import TransformerEncoder
-from .cnn import ConvLayer, ConcatPool, WhiteningConv
+from .cnn import ConvLayer, ConcatPool
 from einops import einsum
 from einops.layers.torch import Rearrange
 import torch.nn as nn
@@ -49,7 +49,7 @@ class CCTEncoder(nn.Module):
     def __init__(
         self,
         image_size=32,
-        conv_kernel_size=3,  # Only 2 is supported for eigenvector initialisation
+        conv_kernel_size=3,
         conv_pooling_type="maxpool",
         conv_pooling_kernel_size=3,
         conv_pooling_kernel_stride=2,
@@ -59,6 +59,11 @@ class CCTEncoder(nn.Module):
         transformer_heads=4,
         transformer_mlp_ratio=2,
         transformer_bos_tokens=4,
+        tranformer_share_kv=True,
+        tranformer_max_subtract=True,
+        tranformer_d_model_scale=True,
+        tranformer_log_length_scale=True,
+        tranformer_quiet_attention=True,
         activation: nn.Module = nn.ReLU,
         activation_kwargs: Optional[dict] = None,
         mlp_dropout=0.0,
@@ -78,6 +83,7 @@ class CCTEncoder(nn.Module):
             self.activation = activation()
 
         self.image_size = image_size
+
         output_size = (
             (image_size + 2 * conv_pooling_kernel_padding - conv_pooling_kernel_size)
             / conv_pooling_kernel_stride
@@ -134,6 +140,11 @@ class CCTEncoder(nn.Module):
                 msa_dropout=msa_dropout,
                 stochastic_depth=stochastic_depth,
                 causal=False,
+                share_kv=tranformer_share_kv,
+                max_subtract=tranformer_max_subtract,
+                d_model_scale=tranformer_d_model_scale,
+                log_length_scale=tranformer_log_length_scale,
+                quiet_attention=tranformer_quiet_attention,
                 linear_module=linear_module,
                 bos_tokens=transformer_bos_tokens,
             )
@@ -180,6 +191,11 @@ class CCT(nn.Module):
         transformer_heads=4,
         transformer_mlp_ratio=2,
         transformer_bos_tokens=4,
+        tranformer_share_kv=True,
+        tranformer_max_subtract=True,
+        tranformer_d_model_scale=True,
+        tranformer_log_length_scale=True,
+        tranformer_quiet_attention=True,
         activation: nn.Module = nn.ReLU,
         activation_kwargs: Optional[dict] = None,
         mlp_dropout=0.0,  # The original paper got best performance from mlp_dropout=0.
@@ -193,17 +209,22 @@ class CCT(nn.Module):
         super().__init__()
 
         self.encoder = CCTEncoder(
-            image_size,
-            conv_kernel_size,
-            pooling_type,
-            pooling_kernel_size,
-            pooling_kernel_stride,
-            pooling_kernel_padding,
-            transformer_embedding_size,
-            transformer_layers,
-            transformer_heads,
-            transformer_mlp_ratio,
-            transformer_bos_tokens,
+            image_size=image_size,
+            conv_kernel_size=conv_kernel_size,
+            conv_pooling_type=pooling_type,
+            conv_pooling_kernel_size=pooling_kernel_size,
+            conv_pooling_kernel_stride=pooling_kernel_stride,
+            conv_pooling_kernel_padding=pooling_kernel_padding,
+            transformer_embedding_size=transformer_embedding_size,
+            transformer_layers=transformer_layers,
+            transformer_heads=transformer_heads,
+            transformer_mlp_ratio=transformer_mlp_ratio,
+            transformer_bos_tokens=transformer_bos_tokens,
+            tranformer_share_kv=tranformer_share_kv,
+            tranformer_max_subtract=tranformer_max_subtract,
+            tranformer_d_model_scale=tranformer_d_model_scale,
+            tranformer_log_length_scale=tranformer_log_length_scale,
+            tranformer_quiet_attention=tranformer_quiet_attention,
             activation=activation,
             activation_kwargs=activation_kwargs,
             mlp_dropout=mlp_dropout,
